/*
    Copyright (C) 2004 Trolltech AS. All Rights Reserved.

    This file ("Example Code"), is part of an example program for Qt,
    and is licensed under the Qt Commercial License Agreement,
    Agreement version 2.4 and higher as well as under the GPL.

    The Example Code is licensed to you "as is" without warranty of
    any kind. To the maximum extent permitted by applicable law,
    Trolltech on behalf of itself and its suppliers, disclaims all
    warranties express or implied, including, but not limited to,
    implied warranties of merchantability and fitness for a particular
    purpose with regard to the Example Code. Trolltech does not
    warrant that the Example Code will satisfy your requirements or
    that it is without defect or error or that the operation thereof
    will be uninterrupted. All use of and reliance on the Example Code
    is at the sole risk of and responsibility of you and your
    customers.

    If you are a holder of a Qt Commercial License Agreement, you can
    use the code under that agreement or you can use it under GPL. If
    you are not a holder of a Qt Commercial License Agreement, you can
    use the code under the GPL. Regardless of the applicable license
    agreement, the Example Code may be used, distributed and modified
    without limitation.
*/

#include <qcolordialog.h>
#include <qinputdialog.h>
#include <qpopupmenu.h>

#include "paletteiconview.h"
#include "palettemodelmanager.h"

PaletteIconView::PaletteIconView(QWidget *parent, const char *name)
    : QIconView(parent, name), theManager(0)
{
    setPaletteModelManager(PaletteModelManager::getInstance());
    connect(this, SIGNAL(contextMenuRequested(QIconViewItem *, const QPoint &)),
            this, SLOT(contextMenuRequested(QIconViewItem *, const QPoint &)));
    setAutoArrange(true);
    setResizeMode(QIconView::Adjust);
    setItemsMovable(false);
    setSorting(true);
    setGridX(75);
    setSelectionMode(QIconView::NoSelection);
    setWordWrapIconText(false);
}

void PaletteIconView::setPaletteModelManager(PaletteModelManager *manager)
{
    if (theManager == manager)
        return; // nothing changes
    if (theManager != 0) {
        // disconnect from old manager
        disconnect(theManager, SIGNAL(colorAdded(const QString &)),
                   this, SLOT(colorAdded(const QString &)));
        disconnect(theManager, SIGNAL(colorChanged(const QString &, const QColor &)),
                   this, SLOT(colorChanged(const QString &, const QColor &)));
        disconnect(theManager, SIGNAL(colorRemoved(const QString &)),
                   this, SLOT(colorRemoved(const QString &)));
        clearOld();
    }
    theManager = manager;
    if (theManager) {
        fillNew();
        connect(theManager, SIGNAL(colorAdded(const QString &)),
                this, SLOT(colorAdded(const QString &)));
        connect(theManager, SIGNAL(colorChanged(const QString &, const QColor &)),
                this, SLOT(colorChanged(const QString &, const QColor &)));
        connect(theManager, SIGNAL(colorRemoved(const QString &)),
                this, SLOT(colorRemoved(const QString &)));
    }
}

void PaletteIconView::colorAdded(const QString &id)
{
    QIconViewItem *item = new QIconViewItem(this, id, getColorPixmap(theManager->getColor(id)));
    itemFromColorId[id] = item;
    colorIdFromItem[item] = id;
}

void PaletteIconView::colorChanged(const QString &id, const QColor &newColor)
{
    QIconViewItem *item = itemFromColorId[id];
    item->setPixmap(getColorPixmap(newColor));
}

void PaletteIconView::colorRemoved(const QString &id)
{
    QIconViewItem *item = itemFromColorId[id];
    delete item;
    itemFromColorId.remove(id);
    colorIdFromItem.remove(item);
    arrangeItemsInGrid();
}

void PaletteIconView::clearOld()
{
    itemFromColorId.clear();
    colorIdFromItem.clear();
    clear();
}

void PaletteIconView::fillNew()
{
    QMap<QString, QColor> palette = theManager->getPalette();
    QMap<QString, QColor>::const_iterator itPalette = palette.constBegin();
    while (itPalette != palette.constEnd()) {
        colorAdded(itPalette.key());
        ++itPalette;
    }
}

QPixmap PaletteIconView::getColorPixmap(const QColor &color) const
{
    QPixmap pix(64, 64);
    pix.fill(color);
    return pix;
}

void PaletteIconView::contextMenuRequested(QIconViewItem *item, const QPoint &pos)
{
    if (!theManager)
        return;
    QPopupMenu menu(this);
    int idAdd = menu.insertItem(tr("Add Color"));
    int idChange = menu.insertItem(tr("Change Color"));
    int idRemove = menu.insertItem(tr("Remove Color"));
    if (!item) {
        menu.setItemEnabled(idChange, false);
        menu.setItemEnabled(idRemove, false);
    }
    int result = menu.exec(pos);

    if (result == idAdd) {
        QColor newColor = QColorDialog::getColor();
        if (newColor.isValid()) {
            QString name = QInputDialog::getText(tr("MVC Palette"), tr("Color Name"));
            if (!name.isEmpty())
                theManager->addColor(name, newColor);
        }
    }
    else if (result == idChange) {
        QString colorId = colorIdFromItem[item];
        QColor oldColor = theManager->getColor(colorId);
        QColor newColor = QColorDialog::getColor(oldColor);
        if (newColor.isValid())
            theManager->changeColor(colorId, newColor);
    }
    else if (result == idRemove) {
        QString colorId = colorIdFromItem[item];
        theManager->removeColor(colorId);
    }
}
